/* utilFileMem.h prototypes for routines in the file memory package */

#ifndef _UTILFILEMEM
#define _UTILFILEMEM

/* -------- PUBLIC INTERFACE --------- */

#define UFM_NEW 1
#define UFM_WRITE 2
#define UFM_READ 3

#ifdef __cplusplus
#define ARGS(X) X
#define EXTERNC extern "C"
#else
#define ARGS(X) ()
#define EXTERNC 
#endif


/* public functions for the package */
EXTERNC int initFileMem ARGS((
  char *filename, /* the name of the memory file to use or create; if NULL use malloc */
  int requestedMode, /* 1  create or overwrite; 2  write existing; 3 readonly */
  int numPages /* when creating this is the size of the file / UFM_BLOCK_SIZE */
  ));

EXTERNC int initGrowableFileMem ARGS((
  char *filename, /* the name of the memory file to use or create; if NULL use malloc */
  int requestedMode, /* 1  create or overwrite; 2  write existing; 3 readonly */
  int numPages, /* when creating: the size of the file / UFM_BLOCK_SIZE (65536) */
  int maxPages /* when creating: largest possible size this DB can grow to */
  ));

EXTERNC char *getFirstBlockOfFileMem();

EXTERNC char *allocFileMem ARGS((int size));

EXTERNC char *reallocFileMem ARGS((
  void *address,
  int size
  ));

EXTERNC void freeFileMem ARGS((void *address));

EXTERNC int terminateFileMem();

EXTERNC int minimizeFileMem();

EXTERNC void debugPrintFileMem();

EXTERNC void debugPrintMem ARGS((void *address));

EXTERNC void debugDumpAll();

EXTERNC void fileMemState ARGS((
  int *initialized,
  int *mode
  ));

/* error codes from functions */
#define UFM_ALREADY_INITIALIZED     1
#define UFM_FILE_NOT_SPECIFIED      2
#define UFM_FILE_NOT_FOUND          3
#define UFM_FILE_NOT_BLOCK_MULTIPLE 4
#define UFM_COULD_NOT_OPEN_FILE     5
#define UFM_COULD_NOT_CREATE_EMPTY  6
#define UFM_COULD_NOT_MALLOC        7
#define UFM_COULD_NOT_MAP           8
#define UFM_COULD_NOT_BLANK_EMPTY   9
#define UFM_MIN_SIZE_IS_ONE_PAGE    10
#define UFM_MAX_LESS_THAN_MIN       11
#define UFM_UNKNOWN_MODE            12
#define UFM_INVALID_OP_ON_READ_ONLY 13
#define UFM_COULD_NOT_MINIMIZE      14

/* information codes from functions */
#define UFM_COULD_NOT_GROW_SPACE_ENOUGH  101
#define UFM_COULD_NOT_GROW_FILE_ENOUGH   102


/* -------- PRIVATE DEFINITIONS --------- */

/* The following is the highest virtual memory address that we can use
   memory will be allocated in the area below this. This figure is designed to
   ensure that the data is put below the minimum possible location of stack space */
#define MAX_MEM_LOCATION (char *)0xef000000

#define UFM_ALLOCATED ((struct memBlock *) -1)

#define UFM_BLOCK_SIZE 65536

#define ROUNDUPTO(POSINT, MULTIPLE) ((((POSINT)-1)/(MULTIPLE))*(MULTIPLE)+(MULTIPLE))


/* structure for one allocated block of memory */
struct memBlock
{
  struct memBlock *next; /* null if first contiguous block */
  struct memBlock *prev; /* null if last contiguous block */
  struct memBlock *nextFree; /* NULL if end of chain; UFM_ALLOCATED if allocated */
  int size; /* size of block including header; subtract to get data size */
  double data; /*forces alignment */
};
       
/* structure for the whole of the memory file */
struct fileMem
{
  struct fileMem *correctFileMemHead; /* correct address of this structure */
  struct memBlock *firstFree; /* start of the chain of free space */
  struct memBlock *firstBlock; /* chain of used space */
  struct memBlock *lastBlock; /* block with the highest address */
};

/* private internal functions */
int growFileMem();
int mapToMemory();
struct memBlock *getActualAddress();

EXTERNC char *allocFileMemRecommended ARGS((
  int size,
  struct memBlock *recommended
  ));

EXTERNC void freeFileMemPossReallocate ARGS((
  void *address,
  int aboutToReallocate
  ));

#endif
